{% extends "base.html" %}

{% block site_meta %}
    {{ super() }}
    <meta name="keywords" content="prompt engineering, prompt management, LLM, AI">
    <meta name="robots" content="index, follow">
    <meta name="language" content="English">
{% endblock %}

{% block content %}
<div class="notebook-links">
  {% if page.nb_url %}
    <a href="{{ page.nb_url }}" title="Download Notebook" class="md-content__button md-icon">
      {% include ".icons/material/download.svg" %}
    </a>
  {% endif %}
</div>

{{ super() }}
{% endblock content %}
